package kieker.webgui.common;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import org.primefaces.model.UploadedFile;

import kieker.analysis.AnalysisController;
import kieker.analysis.model.analysisMetaModel.MIProject;
import kieker.common.logging.Log;
import kieker.common.logging.LogFactory;

/**
 * This is a singleton class for the access to the file system. It makes sure that the necessary directories for the execution of the application exist. <b>Do
 * not</b> remove directories created from this manager during runtime! Directories are created during first access to the class.<br>
 * Currently nearly all methods are synchronized. A fine grained synchronization should be implemented in the future.
 * 
 * @author Nils Christian Ehmke
 */
public final class FileManager {

	private static final Log LOG = LogFactory.getLog(FileManager.class);

	private static final String ROOT_DIR = "data";
	private static final String PROJECT_DIR = ROOT_DIR + File.separator + "projects";
	private static final String LIB_DIR = ROOT_DIR + File.separator + "libraries";
	private static final String EXTENSION = ".xml";
	private static final int BUF_SIZE = 1024;
	private static final FileManager instance = new FileManager();

	private FileManager() {
		checkAndCreateDirectories();
	}

	private synchronized void checkAndCreateDirectories() {
		/*
		 * Make sure that the directories exist and create them if necessary.
		 */
		final File dirProj = new File(PROJECT_DIR);
		final File dirLib = new File(LIB_DIR);
		boolean couldCreated = true;

		if (!dirProj.exists()) {
			couldCreated &= dirProj.mkdirs();
		}
		if (!dirLib.exists()) {
			couldCreated &= dirLib.mkdirs();
		}

		if (!couldCreated) {
			FileManager.LOG.error("Could not create the necessary directories for the application");
		}
	}

	/**
	 * This method saves a given project using the name of the project as the project-directory. The project-directory must already exist.
	 * 
	 * @param project
	 *            The project to be stored.
	 * @return true iff the project-directory does already exist and the storage was successful.
	 */
	public synchronized boolean saveProject(final MIProject project) {
		final String projectName = project.getName();

		final File dirProject = new File(PROJECT_DIR + File.separator + projectName);

		/* Make sure that the directory for the project exists. */
		if (!dirProject.exists()) {
			return false;
		} else {
			/* Try to save the project. */
			final File fileProject = new File(dirProject, projectName + FileManager.EXTENSION);
			try {
				final AnalysisController controller = new AnalysisController(project);
				return controller.saveToFile(fileProject, projectName);
			} catch (final Exception ex) {
				FileManager.LOG.error("Could not save project '" + projectName + "'.");
				return false;
			}
		}
	}

	/**
	 * This method saves a given project using the name of the project as the project-directory. THe project-directory must not already exist.
	 * 
	 * @param project
	 *            The project to be stored.
	 * @return true iff the project-directory does not already exist and the storage was successful.
	 */
	public synchronized boolean saveNewProject(final MIProject project) {
		final String projectName = project.getName();

		final File dirProject = new File(PROJECT_DIR + File.separator + projectName);

		/* Make sure that the project does not already exist and create the project directory. */
		if (dirProject.exists() || !dirProject.mkdir()) {
			return false;
		} else {
			/* The directory should exist now. Store the project. */
			return saveProject(project);
		}
	}

	/**
	 * This method can be used to load all currently saved projects.
	 * 
	 * @return A list containing the loaded projects. If something went wrong, an empty list will be returned, never null.
	 */
	public synchronized List<MIProject> loadAllProjects() {
		List<MIProject> resultList = new ArrayList<MIProject>();

		/* Try to get all directories within the project directory. */
		final File directories[] = new File(PROJECT_DIR).listFiles();
		if (directories != null) {
			for (File directory : directories) {
				if (directory.isDirectory()) {
					/* If there is a project file within the directory, we know the name of it. */
					final File projectFile = new File(directory, directory.getName() + FileManager.EXTENSION);
					if (projectFile.exists()) {
						try {
							/* Try to load the project. */
							MIProject project = AnalysisController.loadFromFile(projectFile);
							if (project != null) {
								resultList.add(project);
							}
						} catch (Exception ex) {
							FileManager.LOG.error("Could not load project '" + directory.getName() + "'.");
						}
					}
				}
			}
		}

		return resultList;
	}

	/**
	 * This method uploads a given file as a new dependency. The file is stored within the lib-directory of this application. If a file with the same name does
	 * already exist, it will be replaces.
	 * 
	 * @param file
	 *            The file to be uploaded.
	 * @return true iff the file has been uploaded successfully.
	 */
	public boolean uploadDependency(final UploadedFile file) {
		System.out.println(file);
		final File depFile = new File(LIB_DIR, file.getFileName());

		InputStream in = null;
		OutputStream out = null;
		try {
			/* Get the streams. */
			in = file.getInputstream();
			out = new FileOutputStream(depFile);
			final byte buf[] = new byte[BUF_SIZE];
			int count;

			/* Transfer the file. */
			while ((count = in.read(buf)) != -1) {
				out.write(buf, 0, count);
			}
			
		} catch (final IOException ex) {
			FileManager.LOG.error("Could not transfer file '" + file.getFileName() + "'");
			return false;
		} finally {
			/* Try to make sure that the streams will be closed. */
			try {
				if (in != null) {
					in.close();
				}
				if (out != null) {
					out.close();
				}
			} catch (IOException ex) {
				// Ignore
			}
		}

		return true;
	}

	/**
	 * Delivers the only instance of this class.
	 * 
	 * @return The singleton instance of this class.
	 */
	public static FileManager getInstance() {
		return instance;
	}
}
